package com.dayou.interceptor;

import cn.hutool.core.util.StrUtil;
import com.dayou.configuration.BusinessException;
import com.dayou.configuration.ErrorCode;
import com.dayou.configuration.JwtConstants;
import com.dayou.configuration.annotation.IgnoreAuth;
import com.dayou.entity.User;
import com.dayou.utils.JwtTokenUtil;
import com.dayou.utils.LoginContext;
import com.google.common.cache.Cache;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.MessageFormat;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/12
 * created with IntelliJ IDEA.
 */
@Component
@Slf4j
public class LoginInterceptor implements HandlerInterceptor {

    @Autowired
    @Qualifier("loginCache")
    private Cache<Long, User> loginCache;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        Boolean ignore = checkIgnore(handler);

        String token = request.getHeader(JwtConstants.REST_TOKEN);

        if (StrUtil.isBlank(token)){
            token = request.getParameter(JwtConstants.REST_TOKEN);
        }
        if (StringUtils.isBlank(token) && !ignore) {
            ErrorCode.throwBusinessException(ErrorCode.UN_LOGIN);
        }

        User user = null;

        try {
            if (StrUtil.isNotBlank(token)){
                Claims claims = JwtTokenUtil.getClaimFromToken(token, JwtConstants.SECRET);
                String subject = claims.getSubject();
                Long userId = Long.valueOf(subject);
                user = loginCache.getIfPresent(userId);
                if (user == null) {
                    ErrorCode.throwBusinessException(ErrorCode.LOGIN_EXPIRE);
                }
                LoginContext.setUser(user);
            }
        } catch (ExpiredJwtException ee){
            //判断token是否过期
            ErrorCode.throwBusinessException(ErrorCode.LOGIN_EXPIRE);
        }
        catch (Exception e) {
            if (e instanceof BusinessException) {
                throw e;
            }
            log.warn(MessageFormat.format("非法token,token:{0}", token));
            ErrorCode.throwBusinessException(ErrorCode.INVALID_TOKEN);
        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {
        LoginContext.removeAll();
    }

    private Boolean checkIgnore(Object handler) {
        if (handler == null){
            return true;
        }
        if (!(handler instanceof HandlerMethod)){
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        IgnoreAuth ignoreAuthOnMethod = handlerMethod.getMethodAnnotation(IgnoreAuth.class);

        if (ignoreAuthOnMethod != null){
            return true;
        }

        Class<?> controllerBean = ((HandlerMethod) handler).getBeanType();
        IgnoreAuth ignoreAuthOnClass = controllerBean.getAnnotation(IgnoreAuth.class);
        if (null != ignoreAuthOnClass) {
            return true;
        }
        return false;
    }
}
